using UnityEngine;
using System.Collections;

public class GameSceneScript : MonoBehaviour
{
	
	public GameObject Car;
	public GameObject GUILoading;
	public GameObject GUIVictoryScreen;
	public GameObject GUITouchToReturn;
	public TextMesh GUIVictoryTimer;
	
	public GUIText lapLabel;
	public GUIText timerLabel;
	public GUIText speedLabel;
	public GUIText accelerationLabel;
	
	private AudioSource _audioSource;
	private float _baseCameraY;
	private float _currentCameraY;
	
	private SimpleAnimator simpleAnimator;
	
	private CarMovementController carMovementController;
	private CheckpointSupervisor checkpointSupervisor;
	private Timer timerScript;
	
	private bool _switchScene;
	private bool _startCoroutine;
	
	private bool _endGame;
	private bool _enableEndGameAnimationAllowed;
	private bool _enableEndGameSwitchScene;
	
	// Use this for initialization
	void Awake ()
	{
		_baseCameraY = transform.localPosition.y;
		_currentCameraY = _baseCameraY;
		_audioSource = GetComponent<AudioSource> ();
		_switchScene = false;
		
		_endGame = false;
		_enableEndGameAnimationAllowed = false;
		
		simpleAnimator = gameObject.GetComponent<SimpleAnimator>();
		
		carMovementController = Car.GetComponent<CarMovementController> ();
		checkpointSupervisor = gameObject.GetComponent<CheckpointSupervisor> ();
		timerScript = gameObject.GetComponent<Timer> ();
	}
	
	// Update is called once per frame
	void Update ()
	{
		// Update speed/acceleration and lap values
		UpdateGui ();
		
		// This one gets us to the main menu
		if (Input.GetKeyDown (KeyCode.Escape) && !_switchScene && !_enableEndGameAnimationAllowed) {
			if (carMovementController.controlsEnabled) {
				// Display "loading" label
				GUILoading.active = true;
				_switchScene = true;
			}
		}
		
		if (_switchScene) {
			// Turn off sound
			AudioListener audioListener = gameObject.GetComponent<AudioListener> ();
			audioListener.enabled = false;
			_audioSource.volume = 0;
			
			Application.LoadLevel ("mainMenu");
		}
		
		// Camera distance regulation based on car acceleration
		_currentCameraY = Mathf.Lerp (_currentCameraY, (_baseCameraY * Mathf.Abs (carMovementController.acceleration / 50)), Time.deltaTime / 4);
		
		if (_baseCameraY > _currentCameraY) {
			_currentCameraY = _baseCameraY;
		} else if (_currentCameraY > 1.5f) {
			_currentCameraY = 1.5f;
		}
		
		// Camera transformation based on car position and speed (thanks to this crap camera follows the car)
		transform.localPosition = new Vector3 (Car.transform.localPosition.x, _currentCameraY, Car.transform.localPosition.z);
		
		// Check if race is finished
		CheckIfFinished ();
	}
	
	void CheckIfFinished ()
	{
		// Check laps
		if (checkpointSupervisor.GetCurrentLap () > checkpointSupervisor.GetMaxLap () && !_endGame) {
			timerScript.StopTimer();
			
			// Disable controls
			carMovementController.controlsEnabled = false;
			//Animate race ending gui gumps
			VectorSet[] vectorSetArray = new VectorSet[2];
			vectorSetArray[0] = new VectorSet (new Vector3 (-7.811752f, 2.695533f, 14.79828f), new Vector3 (-7.811752f, 9.5f, 14.79828f), false);
			vectorSetArray[1] = new VectorSet (new Vector3 (-7.811752f, -0.8531998f, 14.79828f), new Vector3 (-7.811752f, -3.82687f, 14.79828f), false);
			
			GameObject[] gameObjectList = new GameObject[2];
			gameObjectList[0] = GUIVictoryScreen;
			gameObjectList[1] = GUITouchToReturn;
			
			// Setup timer text
			GUIVictoryTimer.text = timerScript.GetMinutes () + ":" + timerScript.GetSeconds () + ":" + timerScript.GetMiliSeconds ();
			
			simpleAnimator.AnimateGroup (vectorSetArray, gameObjectList, 2, false, true);
			
			_endGame = true;
		}
		
		if (_endGame == true && simpleAnimator.IsAnimationComencing () == false) {
			_enableEndGameAnimationAllowed = true;
		}
		
		if (_enableEndGameAnimationAllowed) {
			if ((Input.touchCount > 0 && Input.GetTouch (0).phase == TouchPhase.Ended || Input.GetKeyDown(KeyCode.Space)) 
			    && !_switchScene && simpleAnimator.IsAnimationComencing()  == false) {
				
				VectorSet[] vectorSetArray = new VectorSet[2];
				vectorSetArray[0] = new VectorSet (new Vector3 (-7.811752f, 2.695533f, 14.79828f), new Vector3 (-7.811752f, 9.5f, 14.79828f), true);
				vectorSetArray[1] = new VectorSet (new Vector3 (-7.811752f, -0.8531998f, 14.79828f), new Vector3 (-7.811752f, -3.82687f, 14.79828f), true);
				
				GameObject[] gameObjectList = new GameObject[2];
				gameObjectList[0] = GUIVictoryScreen;
				gameObjectList[1] = GUITouchToReturn;
				
				simpleAnimator.AnimateGroup (vectorSetArray, gameObjectList, 3, true, false);
				
				_enableEndGameAnimationAllowed = false;
				
				_endGame = false;
				
				_enableEndGameSwitchScene = true;
			}
		}
		
		if (_enableEndGameSwitchScene && simpleAnimator.IsAnimationComencing()  == false){
			_enableEndGameSwitchScene = false;
			GUILoading.active = true;
			_switchScene = true;
		}
	}
	
	void UpdateGui ()
	{
		if (Config.DISPLAY_GAME_GUI) {
			// Speed display visual hack
			if (carMovementController.speed > 200) {
				carMovementController.speed = 200;
			}
			
			// Car acceleration display visual hack
			if (carMovementController.acceleration > 200) {
				carMovementController.acceleration = 200;
			}
			
			
			accelerationLabel.text = "Acceleration " + Mathf.Floor (carMovementController.acceleration);
			speedLabel.text = "Speed " + carMovementController.speed;
			if (checkpointSupervisor.GetCurrentLap () > checkpointSupervisor.GetMaxLap ()) {
				lapLabel.text = "Race finished!";
				
			} else {
				lapLabel.text = "Lap " + checkpointSupervisor.GetCurrentLap () + " / " + checkpointSupervisor.GetMaxLap () + " CP: " + checkpointSupervisor.nextCheckpointNumber;
			}
			timerLabel.text = timerScript.GetMinutes () + ":" + timerScript.GetSeconds () + ":" + timerScript.GetMiliSeconds ();
			
			//			GUI.Label (new Rect (50, 70, 130, 30), "Acceleration: " + Mathf.Floor (carMovementController.acceleration));
			//			GUI.Label (new Rect (50, 90, 130, 50), "Speed: " + carMovementController.speed);
		}
	}
}
